unit GWDemoCode;
{---------------------------------------------------------------
Purpose: Implements basic functionality for creating demos that can be
         stepped through.

Author:  Graham Wideman
         graham@wideman-one.com
         www.wideman-one.com

Usage Notes:
------------

Revision History
----------------
98-07-20 GW V1.1 original (version sync'ed with GWDropTarget)

To-do list
----------
-- This unit is still somewhat RefCt demo specific.  If I need to create more
   demos it would be good to factor this better.

-- But then again, if that's the case I need a flexible way to show variable
   values, this demo gets away without that because the diagnostic info is
   displayed by the OLE Monitor window.

---------------------------------------------------------------}
interface
  uses classes, forms, stdctrls, Menus,Grids;
Const
  DemoCountMax = 50;
Type
  //-------------------------------
  TGWDemoCode = class
  //-------------------------------
  protected
    FMenuCaption:  string;
    FFileName   : string;
  public
    constructor Create(AMenuCaption: string; AFileName: string); virtual;
    procedure Pause(S: string);

    //-----------------------------------------------
    // Override the procs below to create actual demo
    //-----------------------------------------------
    procedure Demo_PreRun;   virtual;
    procedure Demo_Run;      virtual;
    procedure Demo_PostRun;  virtual;
  end;

  //-------------------------------
  TGWDemoCode_Mgr = class
  //-------------------------------
  private
    FDemos       : array[0..DemoCountMax-1] of TGWDemoCode;
    FDemoNum     : integer;
    Fmm_Source   : TMemo;
    Flb_FileName : TLabel;
    FHomeDir     : string;

  public
    EndFlag :  Boolean;
    PauseFlag:  Boolean;
    constructor Create;
    function    Add(AGWDemoCode: TGWDemoCode): integer;
    procedure   Init(AMenuItem: TMenuItem; AMenuProc: TNotifyEvent; ALabel: TLabel; AMemo: TMemo);
    function    Demo_Load(Sender: TObject): integer;
    procedure   Demo_Run;
    procedure   Pause(S: string);
    procedure   Step;
  end;

Var
  DemoCode_Mgr: TGWDemoCode_Mgr;

//================================================
implementation
//================================================
uses SysUtils, GWRefCt_Main;

// TGWDemoCode_Mgr;
//------------------------------------
constructor TGWDemoCode_Mgr.Create;
//------------------------------------
Var
  N: integer;
Begin
  inherited Create;
  PauseFlag    := True;
  EndFlag     := False;
  For N := 0 to DemoCountMax-1 do FDemos[N] := nil;
  FDemoNum     := -1;
  Flb_FileName := nil;
  Fmm_Source   := nil;
  FHomeDir     := ExtractFileDir(ParamStr(0)) + '\';
end;

//---------------------------------------------------
procedure TGWDemoCode_Mgr.Init(AMenuItem: TMenuItem; AMenuProc: TNotifyEvent; ALabel: TLabel; AMemo: TMemo);
//---------------------------------------------------
Var
  N: integer;
  mi: TMenuItem;
Begin
  Flb_FileName := ALabel;
  Fmm_Source   := AMemo;

  For N := 0 to DemoCountMax-1 do
  Begin
    if Assigned(FDemos[N]) then
    Begin
      mi         := TMenuItem.Create(fm_Main);
      mi.Caption := FDemos[N].FMenuCaption;
      mi.OnClick := AMenuProc;
      AMenuItem.Add(mi);
    end;
  end;
end;

//---------------------------------------------------
function  TGWDemoCode_Mgr.Add(AGWDemoCode: TGWDemoCode): integer;
//---------------------------------------------------
Var
  N: integer;
Begin
  N := 0;
  While (N < DemoCountMax) and (Assigned(FDemos[N])) do Inc(N);
  If N <> DemoCountMax then
  Begin
    FDemos[N] := AGWDemoCode;
  end;
end;


Const
  StartPause     = 'Waiting to start';
  StartPauseLine = 'Pause(''' + StartPause + ''') ';

//---------------------------------------------------
function TGWDemoCode_Mgr.Demo_Load(Sender: TObject): integer;
//---------------------------------------------------
Var
  DemoCaption: string;
  RowNum: integer;
  S, FilePath: string;
  InFile: TextFile;
  showstate: Boolean;

Label foundit, the_end;
Begin
  DemoCaption := (Sender as TMenuItem).Caption;

  Result := -1;

  Flb_FileName.Caption := '';
  Fmm_Source.Lines.Clear;

  FDemoNum := 0;
  While (FDemoNum < DemoCountMax) do
  Begin
    if Assigned(FDemos[FDemoNum]) then
    Begin
      if FDemos[FDemoNum].FMenuCaption = DemoCaption then goto foundit;
    end;
    Inc(FDemoNum);
  end;
  goto the_end;
foundit:

  // Open file if it exists
  RowNum := 0;
  FilePath := FHomeDir + FDemos[FDemoNum].FFileName;
  if FileExists(FilePath) then
  Begin
    Flb_Filename.Caption := FilePath;
    AssignFile(InFile, FilePath);
    Reset(InFile);

    fmm_Source.Lines.Add(StartPauseLine);

    showstate := false;
    While not EOF(InFile) do
    Begin
      ReadLn(InFile,S);

      if UpperCase(Copy(S,1,9)) = '//SHOWEND' then showstate := false;
      if showstate then
      Begin
        Fmm_Source.Lines.Add(S);
      end else if UpperCase(Copy(S,1,11)) = '//SHOWBEGIN' then showstate := true;
    end;
    CloseFile(InFile);
  end;
  Fmm_Source.Update;
  Fmm_Source.SelStart := 0;
  Fmm_Source.SelLength := 1;
  Result := FDemoNum;
the_end:
end;

//---------------------------------------------------
procedure   TGWDemoCode_Mgr.Demo_Run;
//---------------------------------------------------
Begin
  EndFlag   := false;
  PauseFlag  := true;
  If (FDemoNum >= 0) and (FDemoNum < DemoCountMax) then
  Begin
    if Assigned(FDemos[FDemoNum]) then
    Begin
      FDemos[FDemoNum].Demo_PreRun;
      Pause(StartPause);
      FDemos[FDemoNum].Demo_Run;
      Fmm_Source.SelStart := 0;
      Fmm_Source.SelLength := 1;
      FDemos[FDemoNum].Demo_PostRun;
    end;
  end;
  EndFlag := false;
end;

//---------------------------------------------
procedure TGWDemoCode_Mgr.Pause(S: string);
//---------------------------------------------
Var
  SelStart, SelLength: integer;
  SearchStr: string;
Begin
  SearchStr := 'Pause('''+S+''')';
  If not EndFlag then
  Begin
    // Locate line in memo
    SelLength := Length(SearchStr);
    SelStart  := Pos(SearchStr,Fmm_Source.Lines.Text) - 1;
    Fmm_Source.SelStart  := SelStart;
    Fmm_Source.SelLength := SelLength;

    While PauseFlag and (not EndFlag) do
    Begin
      Application.HandleMessage;
    end;
    PauseFlag := true;
  end;
end;

//---------------------------------------------
procedure TGWDemoCode_Mgr.Step;
//---------------------------------------------
Begin
  PauseFlag := false;
end;

// TGWDemoCode
//---------------------------------------------
constructor TGWDemoCode.Create(AMenuCaption: string; AFileName: string);
//---------------------------------------------
Begin
  inherited Create;
  FMenuCaption := AMenuCaption;
  FFileName    := AFileName;
end;

//---------------------------------------------
procedure TGWDemoCode.Demo_PreRun;
//---------------------------------------------
Begin
end;

//SHOWBEGIN
//---------------------------------------------
procedure TGWDemoCode.Demo_Run;
//---------------------------------------------
Var
  N: integer;
Begin
  // do nothing
                      Pause('a');
  N := 0;             Pause('b');
  N := N + 1;         Pause('c');
end;
//SHOWEND

//---------------------------------------------
procedure TGWDemoCode.Demo_PostRun;
//---------------------------------------------
Begin
end;

//---------------------------------------------
procedure TGWDemoCode.Pause(S: string);
//---------------------------------------------
Begin
  DemoCode_Mgr.Pause(S);
end;


initialization
  DemoCode_Mgr := TGWDemoCode_Mgr.Create;

end.
